@@ -104,6 +104,7 @@ _AUTO_REFRESH_CMDS = [
     'pull', 'merge', 'update', 'push', 'send',
     'conflicts', 'switch', 'bind', 'unbind', 'revert', 'uncommit',
     'shelve', 'unshelve',
+    'ignore',
     # This one really ought to be handled by a custom dialog so that the new
     # location can be opened. Refreshing the current page (often the
     # repository view) is better than nothing though.
@@ -411,6 +412,7 @@ class QExplorerMainWindow(QBzrWindow):
             (self.ui.action_Tag, self.do_bzr_tag),
             (self.ui.action_Plugins, self.do_bzr_plugins),
             (self.ui.action_All_Commands, self.do_bzr_advanced),
+            (self.ui.action_Ignore_Unknowns, self.do_bzr_ignore),
             # Bookmarks ...
             (self.ui.action_Add_Bookmark, self.do_bookmarks_add),
             (self.ui.action_Edit_Bookmarks, self.do_bookmarks_edit_my),
@@ -474,6 +476,7 @@ class QExplorerMainWindow(QBzrWindow):
             self.ui.action_Push:        ['branch', 'bound-branch', 'checkout'],
             self.ui.action_Send:        ['branch', 'bound-branch', 'checkout'],
             self.ui.action_Export:      ['branch', 'bound-branch', 'checkout'],
+            self.ui.action_Tag:         ['branch', 'bound-branch', 'checkout'],
             # Bazaar/Explore ...
             self.ui.action_Browse:      ['branch', 'bound-branch', 'checkout'],
             self.ui.action_Log:         ['branch', 'bound-branch', 'checkout'],
@@ -493,7 +496,7 @@ class QExplorerMainWindow(QBzrWindow):
             self.ui.action_Filter_View: ['working-tree'],
             self.ui.action_Rename:      ['working-tree'],
             self.ui.action_Delete:      ['working-tree'],
-            self.ui.action_Tag:         ['branch', 'bound-branch', 'checkout'],
+            self.ui.action_Ignore_Unknowns: ['working-tree'],
             # Bookmarks
             self.ui.action_Add_Bookmark: ['branch', 'bound-branch', 'checkout',
                 'repository', 'virtual-repository'],
@@ -547,6 +550,7 @@ class QExplorerMainWindow(QBzrWindow):
             self.ui.action_Filter_View: "view",
             self.ui.action_Rename:      "rename",
             self.ui.action_Delete:      "delete",
+            self.ui.action_Ignore_Unknowns: "ignore",
             }
 
     def init_history(self):
@@ -958,11 +962,38 @@ class QExplorerMainWindow(QBzrWindow):
                 mutter("skipping unknown action %s building tool-menu" % name)
 
     def _do_open_tool(self, tool):
+        old_loc_context = []
         if tool.type == 'link':
             link = mod_app_suite.command_to_expanded(tool.action,
                 self._escape_context(self.location_context))
             self.open_web_browser(link)
         else:
+            if  (tool.uses_selected):
+                # populate 'wt_selected' info for context from working tree
+                if isinstance(self._wt_browser._browser,
+                              wt_browser._QBrowseStyleBrowser):
+                    selected_items = \
+                        self._wt_browser._browser._tree_viewer.\
+                        get_selection_items()
+                    if len(selected_items) > 0:
+                        old_loc_context = self.location_context.copy()
+                        paths = [item.path for item in selected_items]
+                        # NOTE:
+                        # Unless items are quoted then filenames with spaces
+                        # in are not handled properly on Windows
+                        escaped = [self._escape_context_item(p, "'")\
+                            for p in paths]
+                        self.location_context['wt_selected'] = " ".join(escaped)
+                elif isinstance(self._wt_browser._browser,
+                                wt_browser._ClassicBrowser):
+                    if self._wt_browser._browser.\
+                        _action_panel._selected_fileinfo:
+                        old_loc_context = self.location_context.copy()
+                        selected_item = self._wt_browser._browser.\
+                            _action_panel._selected_fileinfo.filePath()
+                        self.location_context['wt_selected'] = \
+                            self._escape_context_item(selected_item, "'")
+            #
             args = mod_app_suite.command_to_args(tool.action,
                 self._escape_context(self.location_context))
             if (tool.type == 'bzr') or (tool.type == 'bzr-exec'):
@@ -976,6 +1007,8 @@ class QExplorerMainWindow(QBzrWindow):
                 else:
                     args = ['bzr', 'qrun', '--ui-mode', '--'] + args
             self.run_app(args, use_shell=(tool.type=='shell'))
+            if old_loc_context:
+                self.location_context = old_loc_context.copy()
 
     ## Show message helpers ##
 
@@ -1263,7 +1296,7 @@ class QExplorerMainWindow(QBzrWindow):
         editor = self.desktop_env.get_custom_editor_for(path) or self._editor
         self._record_edit_callback(path, edit_callback)
         try:
-            self.run_app(split_command_options(editor) + [path])
+            self.run_app(split_command_options(editor) + [osutils.abspath(path)])
         except OSError, ex:
             msg = gettext("Failed to edit '%(path)s' using '%(editor)s'.") % \
                     {'path': path, 'editor': editor}
@@ -1467,10 +1500,10 @@ class QExplorerMainWindow(QBzrWindow):
             result[key] = self._escape_context_item(value)
         return result
 
-    def _escape_context_item(self, s):
+    def _escape_context_item(self, s, q='"'):
         """If s contains spaces, surround it with quotes, otherwise return it."""
-        if s and s.find(' ') >= 0:
-            return '"%s"' % (s,)
+        if s and ' ' in s:
+            return '%s%s%s' % (q,s,q)
         else:
             return s
 
@@ -1882,6 +1915,9 @@ class QExplorerMainWindow(QBzrWindow):
         else:
             self.run_bzr_gui("advanced")
 
+    def do_bzr_ignore(self):
+        self.run_bzr_gui("ignore")
+
     ## Bookmarks actions ##
 
     def do_bookmarks_add(self):
